﻿
Imports System.Security.Cryptography

''' <summary>
''' Ver- und Entschlüsselt Bytes mit AES.
''' </summary>
Public Class AesCryptor
    Implements IDisposable

    Dim _Salt As Byte()
    Public ReadOnly Property Salt As Byte()
        Get
            Return _Salt
        End Get
    End Property

    Dim Provider As AesCryptoServiceProvider

    ''' <summary>
    ''' Konstruktor.
    ''' </summary>
    ''' <param name="NewPassword">Das Passwort, mit dem verschlüsselt wird.</param>
    ''' <param name="NewSalt">Ein 16 Bytes langer Salt. Beim Verschlüsseln sollte <see cref="GetRandomSalt"/> verwendet werden, beim Entschlüsseln der gespeicherte Salt.</param>
    Public Sub New(NewPassword As String, NewSalt As Byte())
        If String.IsNullOrEmpty(NewPassword) Then
            Throw New ArgumentNullException("Password")
        End If
        If NewSalt Is Nothing Then
            Throw New ArgumentNullException("Salt")
        End If
        If Not NewSalt.Length = 16 Then
            Throw New ArgumentException("Salt muss 16 Bytes lang sein.", "Salt")
        End If
        _Salt = NewSalt
        Provider = New AesCryptoServiceProvider
        Provider.KeySize = 256
        Provider.Mode = CipherMode.CBC
        Provider.Padding = PaddingMode.PKCS7
        Using Derive As New Rfc2898DeriveBytes(NewPassword, NewSalt, 1000)
            Provider.Key = Derive.GetBytes(32)
            Provider.IV = Derive.GetBytes(16)
        End Using
    End Sub

    ''' <summary>
    ''' Verschlüsselt die angegebenen Bytes.
    ''' </summary>
    ''' <param name="Data">Die zu verschlüsselnden Bytes.</param>
    Public Function Encrypt(Data As Byte()) As Byte()
        Using Encrypter = Provider.CreateEncryptor
            Return Encrypter.TransformFinalBlock(Data, 0, Data.Length)
        End Using
    End Function

    ''' <summary>
    ''' Entschlüsselt die angegebenen Bytes.
    ''' </summary>
    ''' <param name="Data">Die zu entschlüsselnden Bytes.</param>
    ''' <exception cref="ProbablyInvalidPasswordException">Beim Entschlüsseln ist eine <see cref="CryptographicException"/> aufgetreten. Das liegt im Normalfall (sprich: wenn mein Code bugfrei ist) daran, dass ein falsches Passwort angegeben wurde.</exception>
    Public Function Decrypt(Data As Byte()) As Byte()
        Using Encrypter = Provider.CreateDecryptor
            Try
                Return Encrypter.TransformFinalBlock(Data, 0, Data.Length)
            Catch ex As CryptographicException
                Throw New ProbablyInvalidPasswordException(ex)
            End Try
        End Using
    End Function

#Region "IDisposable Support"

    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                Provider.Dispose()
                ' TODO: Verwalteten Zustand löschen (verwaltete Objekte).
            End If

            ' TODO: Nicht verwaltete Ressourcen (nicht verwaltete Objekte) freigeben und Finalize() unten überschreiben.
            ' TODO: Große Felder auf NULL festlegen.
        End If
        Me.disposedValue = True
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

#End Region

    ''' <summary>
    ''' Gibt einen 16 Bytes langen Salt für die Verwendung im Konstruktor zurück, der aus zufälligen Bytes besteht. <see cref="RNGCryptoServiceProvider"/>
    ''' </summary>
    Public Shared Function GetRandomSalt() As Byte()
        Static Rnd As New RNGCryptoServiceProvider
        Dim Temp = New Byte(16 - 1) {}
        Rnd.GetBytes(Temp)
        Return Temp
    End Function

End Class
