﻿
Namespace FileAccess

    ''' <summary>
    ''' Stellt eine Basisklasse für Datei-Accessoren bereit, die Zugriffe auf die Passwords.xml-Datei bereitstellen.
    ''' </summary>
    Public MustInherit Class FileAccessorBase

        ''' <summary>
        ''' Gibt die Dateiversion an, mit der dieser Accessor kompatibel ist.
        ''' </summary>
        Public MustOverride ReadOnly Property Version As Version

        ''' <summary>
        ''' Konvertiert die angegebene Liste an Passwörtern zu einem Xml-Element, um sie in der Passwords.xml-Datei zu speichern.
        ''' </summary>
        ''' <param name="Passwords">Die zu konvertierende Liste an Passwörtern</param>
        ''' <param name="MainPassword">Das Master-Passwort, mit dem die Passwörter verschlüsselt werden.</param>
        Public MustOverride Function PasswordsToXml(Passwords As IEnumerable(Of Password), MainPassword As String) As XElement

        ''' <summary>
        ''' Stellt aus dem angegebenen Xml-Element eine Liste von Passwörtern wieder her.
        ''' </summary>
        ''' <param name="RootElement">Das Xml-Element, das die Passwörter beinhaltet.</param>
        ''' <param name="MainPassword">Das Master-Passwort, mit dem die Passwörter entschlüsselt werden.</param>
        Public MustOverride Function XmlToPasswords(RootElement As XElement, MainPassword As String) As IEnumerable(Of Password)

#Region "Shared"

        ''' <summary>
        ''' Beinhaltet eine Instanz jedes verfügbaren Accessors.
        ''' </summary>
        Private Shared Accessors As IEnumerable(Of FileAccessorBase) = Nothing
        ''' <summary>
        ''' Der Accessor mit der höchsten, verfügbaren Version.
        ''' </summary>
        Private Shared NewestAccessor As FileAccessorBase = Nothing

        ''' <summary>
        ''' Lädt alle verfügbaren Accessoren.
        ''' </summary>
        Private Shared Sub LoadAccessors()
            If Not Accessors Is Nothing Then
                Return
            End If
            Dim Temp As New List(Of FileAccessorBase)
            For Each i In GetType(FileAccessorBase).Assembly.GetTypes
                If i.IsSubclassOf(GetType(FileAccessorBase)) Then
                    Temp.Add(DirectCast(Activator.CreateInstance(i), FileAccessorBase))
                End If
            Next
            If Temp.Count = 0 Then
                Throw New NopeException
            End If
            Accessors = Temp
        End Sub

        ''' <summary>
        ''' Gibt einen Accessor mit der angegebenen Version zurück.
        ''' </summary>
        ''' <param name="Version">Die Version des Accessors.</param>
        ''' <exception cref="FileVersionException">Es wurde kein Accessor mit der angegebenen Version gefunden.</exception>
        Public Shared Function GetCompatibleAccessor(Version As Version) As FileAccessorBase
            LoadAccessors()
            For Each i In Accessors
                If i.Version = Version Then
                    Return i
                End If
            Next
            Throw New FileVersionException("Die Dateiversion ist nicht mit diesem Programm kompatibel.", Version)
        End Function

        ''' <summary>
        ''' Gibt den Accessor mit der höchsten Version zurück.
        ''' </summary>
        Public Shared Function GetNewest() As FileAccessorBase
            LoadAccessors()
            If NewestAccessor Is Nothing Then
                NewestAccessor = Accessors.OrderByDescending(Function(i) i.Version).First
            End If
            Return NewestAccessor
        End Function

#End Region

    End Class

End Namespace
